@@ -1,11 +1,14 @@
-#!/usr/bin/python
+#!/usr/bin/python3.11
 
 import sys
+
+from osgeo.gdal import UseExceptions, deprecation_warn
+
 # import osgeo_utils.gdal_fillnodata as a convenience to use as a script
 from osgeo_utils.gdal_fillnodata import *  # noqa
 from osgeo_utils.gdal_fillnodata import main
-from osgeo.gdal import deprecation_warn
 
+UseExceptions()
 
-deprecation_warn('gdal_fillnodata')
+deprecation_warn("gdal_fillnodata")
 sys.exit(main(sys.argv))
